// --=========================================================================--
//  This confidential and proprietary software may be used only as
//  authorised by a licensing agreement from CAS LAB, NCKU Limited
//    (C) COPYRIGHT 2005 Computer and System LAB, NCKU Limited
//        ALL RIGHTS RESERVED
//  The entire notice above must be reproduced on all authorised
//  copies and copies may only be made to the extent permitted
//  by a licensing agreement from CAS LAB, NCKU Limited.
// 
// -----------------------------------------------------------------------------
// Version and Release Control Information:
// 
// File Name : Write_Buffer_Controller.v,v
// File Revision : 1.2
// 
// Release Information : 
// 
// -----------------------------------------------------------------------------
// Description         : 
// -----------------------------------------------------------------------------
// Revision History    : Revision 1.0  2005/06/09 14:38 Wei-Sheng Lu
//               	 Initial revision
//           		 Revision 1.1  2005/11/16 07:47 Tai-Hua Lu
//               	 Debug list
//                        1.The signals <nWAIT> are missing in the 
//              		sensitivity list of always block.
//            		  2.Signal <Income_pointer_Inc<2>> is assigned but never 
//                          	used.
// 			 Revision 1.2  2007/05/18 08:21 Tai-Hua Lu
//			 The clock triggers change form negative-edge to positive-edge.
// --=========================================================================--

module Write_Buffer_Controller(
    // Signals used during normal operation 
    clk,
    rst_n,
    nWAIT,

    // Signals from CP15 during normal operation
    Drain_WB,

    // Signals from DCache used during Coprocessor operation
    Write_Back,
    Write_Back_Addr,
    Write_Back_Data,
    
    // Signals to DCache used during Coprocessor operation
    Write_Back_nWAIT,

    // Signals from MMU (Data phase) during normal operation
    Write_Buffer_Enable,        
    Data_phase_PA,
            
    // Signals from PipelineReg (Data phase) during normal operation
    Data_phase_nTRANS,
    Data_phase_MAS,
    
    // Signals to ARM9TDMI (Data phase) during normal operation
    WB_nWAIT,
    
    // Signals from ARM9TDMI (Data phase) during normal operation
    DO,

    // Signal from Write Buffer Unit to Wraper during normal operation
    WB_HRDATAM, 
    WB_HREADYM, 
    WB_HADDRM, 
    WB_HTRANSM, 
    WB_HWRITEM, 
    WB_HSIZEM, 
    WB_HBURSTM, 
    WB_HPROTM,
    WB_HWDATAM, 
    WB_HBUSREQM, 
    WB_HLOCKM,
    
    // Signal from Write Buffer Controller to Write buffer RAM during normal operation
    Income_pointer,
    Outgoing_pointer,
    Income_WE,
    Income_Address,
    Income_MAS,
    Income_nTRANS,
    Income_Data,
    Income_Single_Full,
    Outgoing_OE,
    
    // Signal from Write Buffer RAM to Write buffer Controller during normal operation
    Outgoing_Address,
    Outgoing_MAS,
    Outgoing_nTRANS,
    Outgoing_Data,
    Outgoing_Single_Full
);

// Signals used during normal operation 
input       clk;
input       rst_n;
input       nWAIT;

// Signals from CP15 during normal operation
input       Drain_WB;

// Signals from DCache used during Coprocessor operation
input       Write_Back;
//input   [31:4]  Write_Back_Addr;
input   [27:0]  Write_Back_Addr;
input   [127:0] Write_Back_Data;

// Signals to DCache used during Coprocessor operation
output      Write_Back_nWAIT;

// Signals from MMU (Data phase) during normal operation
input       Write_Buffer_Enable;
input   [31:0]  Data_phase_PA;
            
// Signals from PipelineReg (Data phase) during normal operation
input       Data_phase_nTRANS;
input   [1:0]   Data_phase_MAS;
    
// Signals to ARM9TDMI (Data phase) during normal operation
output      WB_nWAIT;
    
// Signals from ARM9TDMI (Data phase) during normal operation
input   [31:0]  DO;
   
// Signals from AMBA bus used during normal operation
input   [31:0]  WB_HRDATAM;
input       WB_HREADYM;
  
// Signals to AMBA bus used during normal operation
output  [31:0]  WB_HADDRM;
output  [1:0]   WB_HTRANSM;
output      WB_HWRITEM;
output  [2:0]   WB_HSIZEM;
output  [2:0]   WB_HBURSTM;
output  [3:0]   WB_HPROTM;
output  [31:0]  WB_HWDATAM;
output      WB_HBUSREQM;
output      WB_HLOCKM;

// Signal from Write Buffer Controller to Write buffer RAM during normal operation
output  [1:0]   Income_pointer;
output  [1:0]   Outgoing_pointer;
output      Income_WE;
output  [31:0]  Income_Address;
output  [1:0]   Income_MAS;
output      Income_nTRANS;
output  [127:0] Income_Data;
output      Income_Single_Full;
output      Outgoing_OE;

// Signal from Write Buffer RAM to Write buffer Controller during normal operation
input   [31:0]  Outgoing_Address;
input   [1:0]   Outgoing_MAS;
input       Outgoing_nTRANS;
input   [127:0] Outgoing_Data;
input       Outgoing_Single_Full;    
  
// Local Signals
wire    [1:0]   delay_counter;
wire    [2:0]   counter_Inc;
wire    [2:0]   Income_pointer_Inc;
wire    [2:0]   Outgoing_pointer_Inc;
wire    [31:0]  Burst_Data;

// Address Counter 
reg [1:0]   counter;

reg [1:0]   Income_pointer;
reg [1:0]   Outgoing_pointer;
  
reg [1:0]   state;
reg [1:0]   next_state;
  
//------------------------------------------------------------------------------
//  Constant declarations
//------------------------------------------------------------------------------
  `define   WB_IDLE     2'h0
  `define   WB_Wait     2'h1
  `define   WB_Write_Back   2'h2
  `define   WB_Finish   2'h3
  
//----------------------------------------------------------------------
// Beginning of main code
//----------------------------------------------------------------------

assign delay_counter        = counter-1;

assign counter_Inc      = counter+1;

assign Income_pointer_Inc   = Income_pointer+1;

assign Outgoing_pointer_Inc = Outgoing_pointer+1;

assign Income_WE        = (~WB_nWAIT)? 1'b0:
                  (Write_Back)? 1'b1:
                  (Write_Buffer_Enable)? 1'b1: 1'b0;
              
assign Income_Address       = (Write_Back)? {Write_Back_Addr,4'b0}:
                  (Write_Buffer_Enable)? Data_phase_PA: 32'b0;
              
assign Income_MAS       = (Write_Back)? 2'b10:  
                  (Write_Buffer_Enable)? Data_phase_MAS :2'b00;
              
assign Income_nTRANS        = (Write_Back)? 1'b1: Data_phase_nTRANS;                    

assign Income_Data      = (Write_Back)? Write_Back_Data:
                  (Write_Buffer_Enable)? {DO,DO,DO,DO}:128'b0;
              
assign Income_Single_Full   = (Write_Back)? 1'b1 :1'b0;          

assign Outgoing_OE      = (state==`WB_IDLE)? 1'b0: 
                  (state==`WB_Wait)? 1'b0: 1'b1;

assign Write_Back_nWAIT     = (~Write_Back)? 1'b1:              
                  (Income_pointer_Inc[1:0]==Outgoing_pointer)? 1'b0:
                  (state!=`WB_IDLE)? 1'b0: 1'b1;

assign WB_nWAIT         = (Drain_WB)? (~WB_HBUSREQM):
                  (~Write_Buffer_Enable)? 1'b1:             
                  (Income_pointer_Inc[1:0]==Outgoing_pointer)? 1'b0:
                  (state!=`WB_IDLE)? 1'b0: 1'b1;

// AHB Interface
assign WB_HADDRM        = (~Outgoing_OE)? 32'b0:
                  (Outgoing_Single_Full)? {Outgoing_Address[31:4],counter,2'b00}:Outgoing_Address;

assign WB_HTRANSM       = (~Outgoing_OE)? 2'b00:
                  (state!=`WB_Write_Back)? 2'b00:
                  (~Outgoing_Single_Full)? 2'b10:
                  (counter==2'b00)? 2'b10:2'b11;
              
assign WB_HWRITEM       = (state==`WB_Write_Back)? 1'b1:1'b0;

assign WB_HSIZEM        = (~Outgoing_OE)? 3'b00:{1'b0,Outgoing_MAS};

assign WB_HPROTM        = (~Outgoing_OE)? 4'b1101:{1'b1,1'b1,Outgoing_nTRANS,1'b1};
  
assign WB_HBURSTM       = (~Outgoing_OE)? 3'b0:
                  (Outgoing_Single_Full)? 3'b011:3'b000;

assign Burst_Data       = (delay_counter[1:0]==2'b00)? Outgoing_Data[31:0]:
                  (delay_counter[1:0]==2'b01)? Outgoing_Data[63:32]:
                  (delay_counter[1:0]==2'b10)? Outgoing_Data[95:64]:Outgoing_Data[127:96];
              
assign WB_HWDATAM       = (~Outgoing_OE)? 32'b0:
                  (Outgoing_Single_Full)? Burst_Data:Outgoing_Data[31:0];

assign WB_HBUSREQM      = (Income_pointer!=Outgoing_pointer)? 1'b1:1'b0;

assign WB_HLOCKM        = 1'b0;

  
// Income Pointer
always@(posedge clk or negedge rst_n)
begin
    if(~rst_n)
    begin
        Income_pointer<=0;
    end
    else if((Write_Back||(Write_Buffer_Enable&&nWAIT))&&(WB_nWAIT&&Write_Back_nWAIT))
    begin
        Income_pointer<=Income_pointer_Inc[1:0];
    end 
end
  
// Outgoing Pointer
always@(posedge clk or negedge rst_n)
begin
    if(~rst_n)
    begin
        Outgoing_pointer<=0;
    end
    else if((state==`WB_Finish)&&(Income_pointer!=Outgoing_pointer)&&(WB_HREADYM))
    begin
        Outgoing_pointer<=Outgoing_pointer_Inc[1:0];
    end 
end  
  
// Address Counter (Address phase) 
always@(posedge clk or negedge rst_n)
begin
    if(~rst_n)
    begin
        counter<=2'b0;
    end
    else if((WB_HREADYM)&&(state==`WB_Write_Back)&&(Outgoing_Single_Full))
    begin
        counter<=counter_Inc[1:0];
    end 
end

// FSM
always@(posedge clk or negedge rst_n)
begin
    if(~rst_n)
    begin
        state<=0;
    end
    else if(WB_HREADYM)
    begin
        state<=next_state;
    end 
end

always @(Outgoing_Single_Full or Write_Back or Write_Buffer_Enable or nWAIT or Income_pointer or Income_pointer_Inc or Outgoing_pointer or counter or state) 
begin
    case (state)
        `WB_IDLE:
        begin
            next_state  = (Income_pointer_Inc[1:0]==Outgoing_pointer)? `WB_Wait:
                      (Write_Back)? `WB_IDLE:
                      (Write_Buffer_Enable&&nWAIT)? `WB_IDLE:
                      (Income_pointer!=Outgoing_pointer)? `WB_Wait:`WB_IDLE;
        end
        `WB_Wait:
        begin
            next_state  = `WB_Write_Back;
        end
        `WB_Write_Back:
        begin
            next_state  = (counter==2'b11)? `WB_Finish:
                      (Outgoing_Single_Full)? `WB_Write_Back:`WB_Finish;
        end
        default: //        `WB_Finish:
        begin
            next_state  = `WB_IDLE;
        end
    endcase
end

endmodule
