// --========================================================================--
//  This confidential and proprietary software may be used only as
//  authorised by a licensing agreement from CAS LAB, NCKU Limited
//    (C) COPYRIGHT 2005 Computer and System LAB, NCKU Limited
//        ALL RIGHTS RESERVED
//  The entire notice above must be reproduced on all authorised
//  copies and copies may only be made to the extent permitted
//  by a licensing agreement from CAS LAB, NCKU Limited.
//
// ----------------------------------------------------------------------------
// Version and Release Control Information:
//
// File Name : Write_Buffer_StatusRAM_FPGA.v,v
// File Revision : 1.1
//
// Release Information : 
//
// ----------------------------------------------------------------------------
// Purpose             : 4 x 4 SRAM.
// -----------------------------------------------------------------------------
// Revision History    : Revision 1.0  2005/06/03 17:32 Wei-Sheng Lu
//  			 Initial revision
// 			 Revision 1.1  2007/05/18 08:21 Tai-Hua Lu
//			 The clock triggers change form negative-edge to positive-edge.
// --========================================================================--
//`timescale 1ns/1ps
//------------------------------------------------------------------------------
// Constant declarations
//------------------------------------------------------------------------------
// This section is used to set the size of the memory that is used.
// If the memory depth is changed, then the address bus A must be able to
//  address the whole memory.

  `define Write_Buffer_StatusADDRWIDTH 2    // Address Width
//  `define Write_Buffer_StatusRAMDEPTH 4     // Memory depth
  `define Write_Buffer_StatusRAMWIDTH 4     // Memory Width
  

module Write_Buffer_StatusRAM (clk, RA, WA, DI, DO, CSn, WEn, OEn);

  input                     clk;
  input  [(`Write_Buffer_StatusADDRWIDTH-1):0]  RA;  // memory address
  input  [(`Write_Buffer_StatusADDRWIDTH-1):0]  WA;  // memory address
  input  [(`Write_Buffer_StatusRAMWIDTH-1):0]   DI;  // memory data I
  output [(`Write_Buffer_StatusRAMWIDTH-1):0]   DO; // memory data O
  input                 CSn; // Chip enable
  input                 WEn; // Write enable
  input                 OEn; // Output enable

//------------------------------------------------------------------------------
// Signal declarations
//------------------------------------------------------------------------------
//  reg  [(`Write_Buffer_StatusRAMWIDTH-1):0] Ram [0:((`Write_Buffer_StatusRAMDEPTH) - 1)];   // Memory register array
  reg  [(`Write_Buffer_StatusRAMWIDTH-1):0] Ram0, Ram1, Ram2, Ram3;
  
  wire [(`Write_Buffer_StatusRAMWIDTH-1):0] Ram_out;
//------------------------------------------------------------------------------
// Beginning of main code
//------------------------------------------------------------------------------

//assign DO = (~CSn & ~OEn & WEn)? Ram[RA]:0;
assign Ram_out = (RA==2'b00)?Ram0:(RA==2'b01)?Ram1:(RA==2'b10)?Ram2:Ram3;
assign DO = (~CSn & ~OEn)? Ram_out:0;

  always@(posedge clk)
  begin
    if (~CSn & ~WEn)
//        begin
        case (WA)
          2'b00: Ram0 <= DI;
          2'b01: Ram1 <= DI;
          2'b10: Ram2 <= DI;
          default: Ram3 <= DI;
        endcase
//        Ram[WA]  <= DI; 
//  end
  end


endmodule

// --================================= End ===================================--
