// --=========================================================================--
//  This confidential and proprietary software may be used only as
//  authorised by a licensing agreement from CAS LAB, NCKU Limited
//    (C) COPYRIGHT 2005 Computer and System LAB, NCKU Limited
//        ALL RIGHTS RESERVED
//  The entire notice above must be reproduced on all authorised
//  copies and copies may only be made to the extent permitted
//  by a licensing agreement from CAS LAB, NCKU Limited.
// 
// -----------------------------------------------------------------------------
// Version and Release Control Information:
// 
// File Name : Write_Buffer_Unit.v,v
// File Revision : 1.1
// 
// Release Information : 
// 
// -----------------------------------------------------------------------------
// Description         : 
// -----------------------------------------------------------------------------
// Revision History    : Revision 1.0  2005/06/09 14:38 Wei-Sheng Lu
//  			 Initial revision
// 			 Revision 1.1  2007/05/18 08:21 Tai-Hua Lu
//			 The clock triggers change form negative-edge to positive-edge.
// --=========================================================================--

module Write_Buffer_Unit(
	// Signals used during normal operation 
	clk,
	rst_n,
	nWAIT,

	// Signals from CP15 during normal operation
	Drain_WB,

	// Signals from DCache used during Coprocessor operation
	Write_Back,
	Write_Back_Addr,
	Write_Back_Data,
	
	// Signals to DCache used during Coprocessor operation
	Write_Back_nWAIT,

	// Signals from MMU (Data phase) during normal operation
	Write_Buffer_Enable,		
	Data_phase_PA,
			
	// Signals from PipelineReg (Data phase) during normal operation
	Data_phase_nTRANS,
	Data_phase_MAS,
	
	// Signals to ARM9TDMI (Data phase) during normal operation
	WB_nWAIT,
	
	// Signals from ARM9TDMI (Data phase) during normal operation
	DO,

	// Signal from Write Buffer Unit during normal operation
	WB_HRDATAM, 
	WB_HREADYM, 
	WB_HADDRM, 
	WB_HTRANSM, 
	WB_HWRITEM, 
	WB_HSIZEM, 
	WB_HBURSTM, 
	WB_HPROTM,
	WB_HWDATAM, 
	WB_HBUSREQM, 
	WB_HLOCKM
);

// Signals used during normal operation 
input		clk;
input		rst_n;
input		nWAIT;

// Signals from CP15 during normal operation
input		Drain_WB;

// Signals from DCache used during Coprocessor operation
input		Write_Back;
input	[27:0]	Write_Back_Addr;//[31:4]
input	[127:0]	Write_Back_Data;

// Signals to DCache used during Coprocessor operation
output		Write_Back_nWAIT;

// Signals from MMU (Data phase) during normal operation
input		Write_Buffer_Enable;
input	[31:0]	Data_phase_PA;
			
// Signals from PipelineReg (Data phase) during normal operation
input		Data_phase_nTRANS;
input	[1:0]	Data_phase_MAS;
	
// Signals to ARM9TDMI (Data phase) during normal operation
output		WB_nWAIT;
	
// Signals from ARM9TDMI (Data phase) during normal operation
input	[31:0]	DO;
   
// Signals from AMBA bus used during normal operation
input 	[31:0]  WB_HRDATAM;
input		WB_HREADYM;
  
// Signals to AMBA bus used during normal operation
output	[31:0]  WB_HADDRM;
output	[1:0] 	WB_HTRANSM;
output		WB_HWRITEM;
output	[2:0] 	WB_HSIZEM;
output	[2:0] 	WB_HBURSTM;
output	[3:0] 	WB_HPROTM;
output	[31:0]  WB_HWDATAM;
output 		WB_HBUSREQM;
output 		WB_HLOCKM;
  
// Local Signals
wire	[1:0]	Income_pointer;
wire	[1:0]	Outgoing_pointer;
wire		Income_WE;
wire	[31:0]	Income_Address;
wire	[1:0]	Income_MAS;
wire		Income_nTRANS;
wire	[127:0] Income_Data;
wire		Income_Single_Full;

wire		Outgoing_OE;
wire	[31:0]	Outgoing_Address;
wire	[1:0]	Outgoing_MAS;
wire		Outgoing_nTRANS;
wire	[127:0] Outgoing_Data;
wire		Outgoing_Single_Full;	


//----------------------------------------------------------------------
// Beginning of main code
//----------------------------------------------------------------------

Write_Buffer_Addr_unit uWrite_Buffer_Addr_unit(
	// Signals used during normal operation 
	.clk	(clk),
	
	// Signals from Outgoing Controller used during normal operation
	.RAddr	(Outgoing_pointer),
	.OE	(Outgoing_OE),
	.RData	(Outgoing_Address),
	
	// Signals from Income Controller used during normal operation
	.WAddr	(Income_pointer),
	.WE	(Income_WE),
	.WData	(Income_Address)
);

Write_Buffer_Status_unit uWrite_Buffer_Status_unit(
	// Signals used during normal operation 
	.clk	(clk),
	
	// Signals from Outgoing Controller used during normal operation
	.RAddr	(Outgoing_pointer),
	.OE	(Outgoing_OE),
	.RData	({Outgoing_MAS,Outgoing_nTRANS,Outgoing_Single_Full}),
	
	// Signals from Income Controller used during normal operation
	.WAddr	(Income_pointer),
	.WE	(Income_WE),	
	.WData	({Income_MAS,Income_nTRANS,Income_Single_Full})
);
  
Write_Buffer_Data_unit uWrite_Buffer_Data_unit(
	// Signals used during normal operation 
	.clk	(clk),
	
	// Signals from Outgoing Controller used during normal operation
	.RAddr	(Outgoing_pointer),
	.OE	(Outgoing_OE),
	.RData	(Outgoing_Data),
	
	// Signals from Income Controller used during normal operation
	.WAddr	(Income_pointer),
	.WE	(Income_WE),
	.WData	(Income_Data)
);  
  
Write_Buffer_Controller uWrite_Buffer_Controller(
	// Signals used during normal operation 
	.clk			(clk),
	.rst_n			(rst_n),
	.nWAIT			(nWAIT),

	// Signals from CP15 during normal operation
	.Drain_WB		(Drain_WB),

	// Signals from DCache used during Coprocessor operation
	.Write_Back		(Write_Back),
	.Write_Back_Addr	(Write_Back_Addr),
	.Write_Back_Data	(Write_Back_Data),
	
	// Signals from Write Buffer to DCache used during Coprocessor operation
	.Write_Back_nWAIT	(Write_Back_nWAIT),

	// Signals from MMU (Data phase) during normal operation
	.Write_Buffer_Enable	(Write_Buffer_Enable),		
	.Data_phase_PA		(Data_phase_PA),
			
	// Signals from PipelineReg (Data phase) during normal operation
	.Data_phase_nTRANS	(Data_phase_nTRANS),
	.Data_phase_MAS		(Data_phase_MAS),
	
	// Signals to ARM9TDMI (Data phase) during normal operation
	.WB_nWAIT		(WB_nWAIT),
	
	// Signals from ARM9TDMI (Data phase) during normal operation
	.DO			(DO),

	// Signal from Write Buffer Unit to Wraper during normal operation
	.WB_HRDATAM		(WB_HRDATAM), 
	.WB_HREADYM		(WB_HREADYM), 
	.WB_HADDRM		(WB_HADDRM), 
	.WB_HTRANSM		(WB_HTRANSM), 
	.WB_HWRITEM		(WB_HWRITEM), 
	.WB_HSIZEM		(WB_HSIZEM), 
	.WB_HBURSTM		(WB_HBURSTM), 
	.WB_HPROTM		(WB_HPROTM),
	.WB_HWDATAM		(WB_HWDATAM), 
	.WB_HBUSREQM		(WB_HBUSREQM), 
	.WB_HLOCKM		(WB_HLOCKM),
	
	// Signal from Write Buffer Controller to Write buffer RAM during normal operation
	.Income_pointer		(Income_pointer),
	.Outgoing_pointer	(Outgoing_pointer),
	.Income_WE		(Income_WE),
	.Income_Address		(Income_Address),
	.Income_MAS		(Income_MAS),
	.Income_nTRANS		(Income_nTRANS),
	.Income_Data		(Income_Data),
	.Income_Single_Full	(Income_Single_Full),
	.Outgoing_OE		(Outgoing_OE),
	
	// Signal from Write Buffer RAM to Write buffer Controller during normal operation
	.Outgoing_Address	(Outgoing_Address),
	.Outgoing_MAS		(Outgoing_MAS),
	.Outgoing_nTRANS	(Outgoing_nTRANS),
	.Outgoing_Data		(Outgoing_Data),
	.Outgoing_Single_Full	(Outgoing_Single_Full)
);

endmodule