//+FHDR----------------------------------------------------------------
// (C) Copyright CASLab.EE.NCKU
// All Right Reserved
//---------------------------------------------------------------------
// FILE NAME: sram256Kx32.v
// AUTHOR: Chen-Chien Wang
// CONTACT INFORMATION: ccwang@casmail.ee.ncku.edu.tw
//---------------------------------------------------------------------
// RELEASE VERSION: 1.0.0
// VERSION DESCRIPTION: First Edition no errata
//---------------------------------------------------------------------
// RELEASE: 2009/2/5 PM. 05:43:22
//-FHDR----------------------------------------------------------------

// synopsys translate_off
//`include "timescale.v"
// synopsys translate_on

module sram256Kx32(
            CLK,
            ADDR,
            DIN,
            WE,
            DOUT
        );

    //-------------------------------------------------------------
    // Input/Output
    //-------------------------------------------------------------

    input           CLK;
    input   [14:0]  ADDR;
    input   [31:0]  DIN;
    input   [3:0]   WE;
    output  [31:0]  DOUT;

    //-------------------------------------------------------------
    // Define or Parameter
    //-------------------------------------------------------------
  parameter MemSize  = 1;            // Memory size in Kbytes
  parameter FileName = ""; // Input filename
  parameter BaseAddr = 32'h0 ;
    //-------------------------------------------------------------
    // Internal Wire
    //-------------------------------------------------------------
    reg     [31:0]  Mem [0:(256-1)];    // Memory register array
    reg     [31:0]  rdata;
    reg     [31:0]  wdata;
    integer         i;      // Loop counter used in memory initialisation

//=====================================================================
//      Main Body
//=====================================================================

    //-------------------------------------------------------------
    // initial memory
    //-------------------------------------------------------------
  /*  initial
    begin

        for (i=0; i<256*1024; i = i+1)
            Mem[i] = 32'h0000_0000;

    end
	*/
initial
  begin
    $display("### Loading internal memory, Based Addr = 0x%x, Length = 0x%x ###",BaseAddr,MemSize*1024);
    for (i=0; i<=((MemSize * 256)-1); i = i+1)
      Mem[i] = 32'h0000_0000;
    if (FileName != "")
    begin
      $readmemh(FileName, Mem);
    end
    $display("### Load internal memory Complete ###");
  end


    //-------------------------------------------------------------
    // Read
    //-------------------------------------------------------------

    always@(ADDR)
    begin
        rdata = Mem[ADDR];
    end

    assign DOUT = rdata;

    //-------------------------------------------------------------
    // Write
    //-------------------------------------------------------------
    always@(posedge CLK)
    begin

        if(WE!=4'b0000)
        begin

            wdata = Mem[ADDR];

            if(WE[3]) wdata[31:24] =  DIN[31:24] ;
            if(WE[2]) wdata[23:16] =  DIN[23:16] ;
            if(WE[1]) wdata[15:8]  =  DIN[15:8]  ;
            if(WE[0]) wdata[7:0]   =  DIN[7:0]   ;

            Mem[ADDR] = wdata;

        end

    end

endmodule
